package com.alihealth.d2d.provtest.utils;

import com.alihealth.d2d.provtest.config.CertConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.security.*;
import java.security.interfaces.RSAPrivateKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;

/**
 * @author Jason 证书管理
 * @date 2025年06月06日 14:24:09
 */
@Component
public class CertManager {

    private static volatile String privateKey;

    private static volatile String publicKey;

    @Autowired
    private CertConfig certConfig;


    /**
     * 私钥解密
     * @param key
     * @param data
     * @return
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeyException
     * @throws NoSuchPaddingException
     * @throws InvalidKeySpecException
     * @throws BadPaddingException
     * @throws IllegalBlockSizeException
     * @throws IOException
     */
    public  String testDecrypt(String key,String data) throws NoSuchAlgorithmException, InvalidKeyException, NoSuchPaddingException, InvalidKeySpecException, IllegalBlockSizeException, BadPaddingException, IOException{
        byte[] decode = java.util.Base64.getDecoder().decode(key);
        RSAPrivateKey priKey = (RSAPrivateKey) KeyFactory.getInstance(certConfig.getAlgorithm()).generatePrivate(new PKCS8EncodedKeySpec(decode));
        //RSA解密
        Cipher ci = Cipher.getInstance(certConfig.getAlgorithm());
        ci.init(Cipher.DECRYPT_MODE, priKey);


        byte[] bytes = java.util.Base64.getDecoder().decode(data);
        int inputLen = bytes.length;
        int offLen = 0;
        int i = 0;
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        while(inputLen - offLen > 0){
            byte[] cache;
            if(inputLen - offLen > 128){
                cache = ci.doFinal(bytes,offLen,128);
            }else{
                cache = ci.doFinal(bytes,offLen,inputLen - offLen);
            }
            byteArrayOutputStream.write(cache);
            i++;
            offLen = 128 * i;

        }
        byteArrayOutputStream.close();
        byte[] byteArray = byteArrayOutputStream.toByteArray();
        return new String(byteArray);
    }


    @PostConstruct
    public synchronized void init() throws Exception {
        if (privateKey == null) {
            privateKey = initPrivateKey();
        }
        if(publicKey == null) {
            publicKey = initPublicKey();
        }
    }




    /**
     * 获取私钥
     * @return
     * @throws Exception
     */
    public String initPrivateKey() throws Exception {
        KeyStore keyStore = KeyStoreUtil.loadKetStore(certConfig.getPath(),certConfig.getPassword());
        return Base64.encode(KeyStoreUtil.getPrivateKey(keyStore, certConfig.getPassword()).getEncoded());
    }

    public String getPrivateKey() {
        if (privateKey == null) {
            throw new IllegalStateException("获取私钥失败,证书未初始化");
        }
        return privateKey;
    }

    /**
     * 获取公钥
     * @return
     * @throws Exception
     */
    public  String initPublicKey() throws Exception {
        KeyStore keyStore = KeyStoreUtil.loadKetStore(certConfig.getPath(),certConfig.getPassword());
        PublicKey publicKey = KeyStoreUtil.getPublicKey(keyStore);
        return Base64.encode(publicKey.getEncoded());
    }

    public String getPublicKey() {
        if (publicKey == null) {
            throw new IllegalStateException("获取公钥失败，证书未初始化");
        }
        return publicKey;
    }
}
